import pytest
import requests_mock

from ..http import BackofficeHTTPException


def test_order_url(backoffice):
    with requests_mock.mock() as m:
        m.post('http://testhost/orders/', status_code=201, json={'mock': 'mmmock'})
        assert backoffice._post_order('some stuff') == {'mock': 'mmmock'}


def test_non_201_response_to_order_post(backoffice):
    with requests_mock.mock() as m:
        m.post('http://testhost/orders/', status_code=403)
        with pytest.raises(BackofficeHTTPException):
            backoffice._post_order('some stuff')


def test_items_url(backoffice):
    with requests_mock.mock() as m:
        m.post('http://testhost/orders/100500/items/', status_code=201, json={'mock': 'mmmock'})
        assert backoffice._post_items({'id': 100500}, [{'mo': 'ck'}]) == {'mock': 'mmmock'}


def test_non_201_response_to_items_post(backoffice):
    with requests_mock.mock() as m:
        m.post('http://testhost/orders/100500/items/', status_code=403)
        with pytest.raises(BackofficeHTTPException):
            backoffice._post_items({'id': 100500}, [{'mo': 'ck'}])



import pytest

from ..orders import BackofficeOrderValidator, BackofficeValidationError

validator = BackofficeOrderValidator()


@pytest.mark.parametrize('data', (
    {'customer': {
        'name': 'Petrovich',
    }},
    {'customer': {
        'name': 'Petrovich',
        'legal': {'name': 'Petrovich corp.', 'inn': '123456'},
    }},
    {'customer': {
        'name': 'Petrovich',
        'legal': {'name': 'Petrovich corp.', 'inn': '123456'},
    }, 'is_confirmed': True},
    {
        'customer': {
            'name': 'Petrovich',
            'legal': {'name': 'Petrovich corp.', 'inn': '123456'},
        },
        'delivery': {
            'desired_date': '2018-01-01',
            'time_from': '18:01',
            'time_till': '23:39',
        },
    },
    {
        'customer': {'name': 'Petrovich'},
        'delivery': {'floor': '1'},
    },
    {
        'customer': {'name': 'Petrovich'},
        'delivery': {'floor': 1},
    },
    {
        'customer': {'name': 'Petrovich'},
        'delivery': {'floor': '-1'},
    },
    {
        'customer': {'name': 'Petrovich'},
        'delivery': {'floor': -1},
    },
))
def test_valid_order(data):
    assert validator.validate_order(data) is True


@pytest.mark.parametrize('data', (
    {},
    {'customer': {
        'name': 'Petrovich',
        'legal': {'name': 'Petrovich corp.'},
    }},
    {'customer': {
        'name': 'Petrovich',
        'legal': {'name': 'Petrovich corp.', 'inn': '123456'},
    }, 'is_confirmed': None},
    {
        'customer': {
            'name': 'Petrovich',
        },
        'delivery': {
            'desired_date': 'some awesome date',
        },
    },
    {
        'customer': {'name': 'Petrovich'},
        'delivery': {
            'time_from': 'HH:MM',
        },
    },
    {
        'customer': {'name': 'Petrovich'},
        'delivery': {
            'time_till': 'HH:MM',
        },
    },
))
def test_invalid_order(data):
    with pytest.raises(BackofficeValidationError) as e:
        validator.validate_order(data)

        assert "customer" in str(e)


def test_any_utm_object_is_accepted():
    assert validator.validate_order({
        'customer': {
            'name': 'Petrovich',
        },
        'utm': {
            'a': ['b', 'c', {'d': 'e'}],
            'f': 'g',
            'foo': 'bar',
        },
    }) is True


@pytest.mark.parametrize('data', [
    [{'product': {
        'name': 'kamaz of ships',
    }}],
    [{
        'product': {
            'name': 'kamaz of ships',
        },
        'price': 100500.00,
    }],
    [{
        'product': {
            'name': 'kamaz of ships',
        },
        'price': 100500.00,
        'customer_comment': 'Sudo make me a burger',
    }, {
        'product': {
            'name': 'gazel of ships',
        },
        'price': 100500.00,
    }],
])
def test_valid_item_set(data):
    assert validator.validate_items(data) is True


def test_string_items_are_accepted_instead_of_numbers_because_django_deals_correctly_with_it_and_i_dont_wont_to_fucken_mess_with_this_javascript_types():
    assert validator.validate_items([
        {
            'product': {
                'site_id': '100500',
            },
            'price': '1005.05',
        },
    ]) is True


def test_invalid_item_set():
    with pytest.raises(BackofficeValidationError) as e:
        validator.validate_items([
            {
                'quant1ty': 100500,
            },
        ])

        assert "name" in str(e)



import pytest


@pytest.mark.parametrize('input, param, value, expected', [
    ['/test/create', 'par1', 'val1', '/test/create?par1=val1'],
    ['/test/create?par1=DELETED', 'par1', 'val1', '/test/create?par1=val1'],  # overwriting params
    ['/test/create/?par2=val2', 'par1', 'val1', '/test/create/?par2=val2&par1=val1'],
])
def test_append_qs(input, param, value, expected, backoffice):
    assert backoffice.http._append_qs(input, param, value) == expected



import pytest
import requests_mock

from ..http import BackofficeHTTPException


@pytest.mark.parametrize('api_data', (
    {
        'price_cashless': '300500.00',
        'price_cash': '200500.00',
    },
))
def test_price_url(api_data, backoffice):
    with requests_mock.mock() as m:
        m.get('http://testhost/price_lists/sell/100500/', status_code=200, json=api_data)

        assert backoffice.get_price(100500) == api_data


def test_non_200_response_to_price_get(backoffice):
    with requests_mock.mock() as m:
        m.get('http://testhost/price_lists/sell/100500/', status_code=404)

        with pytest.raises(BackofficeHTTPException):
            assert backoffice.get_price(100500)




import json
from os import path
from typing import List

from jsonschema import validate
from jsonschema.exceptions import ValidationError


class BackofficeValidationError(BaseException):
    pass


class BackofficeOrderValidator:
    @staticmethod
    def _get_schema(schema: str):
        filename = path.join(path.dirname(__file__), 'schemas', schema + '.schema.json')

        with open(filename, 'r') as f:
            return json.load(f)

    @classmethod
    def validate_order(cls, order: dict):
        try:
            validate(order, cls._get_schema('order'))

        except ValidationError as e:
            raise BackofficeValidationError(str(e))

        return True

    @classmethod
    def validate_items(cls, items: List):
        try:
            validate(items, cls._get_schema('items'))

        except ValidationError as e:
            raise BackofficeValidationError(str(e))

        return True



from collections import OrderedDict
from typing import Generator
from urllib.parse import parse_qsl, urljoin, urlparse, urlunparse

import requests


class BackofficeHTTPException(BaseException):
    pass


class BackofficeHTTP:
    headers = {
        'Authorization': 'Token {token}',
        'Content-Type': 'application/json',
        'Accept': 'application/json',
    }

    def __init__(self, host: str, token: str, timeout=6):
        self.host = host
        self.timeout = timeout
        self.headers['Authorization'] = self.headers['Authorization'].format(token=token)

    @staticmethod
    def _append_qs(url: str, key: str, value: str) -> str:
        """Append a parameter to the url querystring"""
        url = list(urlparse(url))
        query = OrderedDict(parse_qsl(url[4]))
        query[key] = value
        url[4] = '&'.join(f'{p}={v}' for p, v in query.items())

        return urlunparse(url)

    def paginate(self, url) -> Generator[dict, None, None]:
        next_page = self._append_qs(url, 'page', 1)
        while next_page is not None:
            response = self.get(next_page)

            for result in response.get('results', []):
                yield result

            next_page = response.get('next')

    def get(self, url: str):
        url = urljoin(self.host, url)
        if not url.endswith('/') and '?' not in url:
            url += '/'

        r = requests.get(url, headers=self.headers, timeout=self.timeout)

        if r.status_code != 200:
            raise BackofficeHTTPException(
                'Non-200 response when fetching url {}: {}'.format(url, r.status_code))

        return r.json()

    def post(self, url: str, payload: dict):
        url = urljoin(self.host, url)

        if not url.endswith('/'):
            url += '/'

        r = requests.post(url, json=payload, headers=self.headers, timeout=self.timeout)

        if r.status_code != 201:
            raise BackofficeHTTPException(
                'Non-201 response when posting to url {}: {}'.format(url, r.status_code))

        return r.json()



import pytest
import requests_mock


@pytest.fixture(autouse=True)
def mock_http_response(response):
    with requests_mock.mock() as m:
        m.get('http://testhost/products?page=1', json=response('products_1'))
        m.get('http://testhost/products?page=1', json=response('products_2'))

        yield


def test_ok(backoffice, response):
    got = list(backoffice.http.paginate('products'))

    assert len(got) == 25



"""
The processors work like a hybrid of pytest fixtures and middleware.

To define an order processor, you should create a function with a signle argument 'order'
To defined an items list process, you should create a function with a single argument 'items'

All processors are executed sequentially
"""
from collections import Iterable


def p01_format_date(order):
    """
    Format a russian date dd.mm.yyyy to computer-readble yyyy-mm-dd.

    Attention! This function dropes unparsable dates
    """
    def process(date):
        if '-' in date and len(date) == 10:
            return date

        if '.' in date and len(date) == 10:  # russian format dd.mm.yyyy to yyyy-mm-dd
            dmy = date.split('.')
            if len(dmy) == 3 and all(v is not None for v in dmy):
                return '-'.join(reversed(dmy))

    if 'delivery' in order and 'desired_date' in order['delivery']:
        order['delivery']['desired_date'] = process(order['delivery']['desired_date'])

    return order


def p99_drop_keys_without_values(order):
    for key, value in order.copy().items():
        if isinstance(value, dict):
            order[key] = p99_drop_keys_without_values(value)

        if value is None or (isinstance(value, Iterable) and len(value) == 0):
            del order[key]

    return order



import inspect
from typing import Callable, Generator, Iterable, List, Union

from . import processors
from .http import BackofficeHTTP
from .orders import BackofficeOrderValidator


class BackofficeException(BaseException):
    pass


class BackofficeClient:
    def __init__(self, url: str, token: str):
        self.validator = BackofficeOrderValidator()
        self.processor = BackofficePreprocessor()
        self.http = BackofficeHTTP(url, token)

    def send(self, order: dict, items: List):
        """Preprocess, validatate and send an order to the backoffice API"""
        order = self.processor.run_processors(order, 'order')
        self.validator.validate_order(order)

        order = self._post_order(order)
        if 'id' not in order.keys():
            raise BackofficeException('Incorrect response from backoffice (no order id)')

        items = self.processor.run_processors(items, 'items')
        self.validator.validate_items(items)
        self._post_items(order, items)
        return order

    def _post_order(self, order: dict) -> dict:
        """Send order to the backoffice"""
        return self.http.post('orders', order)

    def _post_items(self, order: dict, items: List):
        """Send order items to the backoffice, given an order"""
        return self.http.post('orders/{order_id}/items/'.format(order_id=order['id']), items)

    def get_price(self, site_id: int) -> dict:
        """Get price from the price list"""
        return self.http.get('price_lists/sell/{site_id}/'.format(site_id=site_id))

    def get_products_sales_stat(self, site_ids: List[Union[str, int]]) -> Iterable:
        """Get product sales statistics"""
        return self._get_data_by_site_ids('products/sales_stat/?site_ids={site_ids}', site_ids)

    def get_products_offers_stat(self, site_ids: List[Union[str, int]]) -> Iterable:
        """Get partner offer statistics"""
        return self._get_data_by_site_ids('products/offers_stat/?site_ids={site_ids}', site_ids)

    def get_offers(self, site_ids: List[Union[str, int]], cashless=False) -> Iterable:
        """Get current offers of all partners (for the warehouse)"""
        if cashless:
            url = 'products/offers/?cashless=1&site_ids={site_ids}'
        else:
            url = 'products/offers/?site_ids={site_ids}'

        return self._get_data_by_site_ids(url, site_ids)

    def _get_data_by_site_ids(self, url, site_ids: List[Union[str, int]]) -> dict:
        site_ids_str = ','.join([str(el) for el in site_ids])

        return self.http.get(url.format(url, site_ids=site_ids_str))


class BackofficePreprocessor:
    @classmethod
    def run_processors(cls, obj, what: str):
        """Runs the processors defined in processors.py.

        What may be an 'order' or 'item'
        """
        for processor in cls.get_processors(what):
            obj = processor(obj)
            if obj is None:
                raise AttributeError(f'Processor {processor} returned nothing!')

        return obj

    @staticmethod
    def get_processors(what: str) -> Generator[Callable, None, None]:
        for member in inspect.getmembers(processors, lambda member: inspect.isfunction(member) and what in inspect.signature(member).parameters.keys()):
            yield member[1]



import pytest
import requests_mock

from ..http import BackofficeHTTPException


@pytest.mark.parametrize('url', [
    '/partners/',
    '/partners',
    'partners/',
    'partners',
])
def test_200(url, backoffice):
    with requests_mock.mock() as m:
        m.get('http://testhost/partners/', status_code=200, json={'__mocked': True})

        assert backoffice.http.get(url) == {'__mocked': True}


def test_404(backoffice):
    with requests_mock.mock() as m:
        m.get('http://testhost/partners/', status_code=404)

        with pytest.raises(BackofficeHTTPException):
            backoffice.http.get('partners')



import json

import pytest

from .client import BackofficeClient


@pytest.fixture
def backoffice():
    return BackofficeClient(
        url='http://testhost',
        token='tsttkn',
    )


@pytest.fixture
def response():
    """Fixture reader"""
    def read_file(fname):
        with open('./backoffice/tests/fixtures/' + fname + '.json') as fp:
            return json.load(fp)

    return read_file



import pytest
import requests_mock

from ..http import BackofficeHTTPException


@pytest.mark.parametrize('api_data', (
    [
        {
            'id': 100,
            'site_id': 128,
            'name': 'Камаз с пирожками',
            'offers': [
                {
                    'partner_id': 'b',
                    'price': 100500,
                },
                {
                    'partner_id': 'd',
                    'price': 100501,
                },
            ],
        },
        {
            'id': 234,
            'site_id': 256,
            'name': 'Ведро с тестами',
            'offers': [
                {
                    'partner_id': 'e',
                    'price': 100503,
                },
                {
                    'partner_id': 'f',
                    'price': 100504,
                },
            ],
        },
    ],
))
def test_url(api_data, backoffice):
    with requests_mock.mock() as m:
        m.get('http://testhost/products/offers/?site_ids=128,256', status_code=200, json=api_data)

        assert backoffice.get_offers([128, 256]) == api_data


def test_cashless(backoffice):
    with requests_mock.mock() as m:
        m.get('http://testhost/products/offers/?cashless=1&site_ids=128,256', status_code=200, json={'a': 'b'})

        assert backoffice.get_offers([128, 256], cashless=True) == {'a': 'b'}


def test_non_200_response_to_product_offers_stat_get(backoffice):
    with requests_mock.mock() as m:
        m.get('http://testhost/products/offers/?site_ids=128,256', status_code=404)

        with pytest.raises(BackofficeHTTPException):
            assert backoffice.get_offers([128, 256])



import pytest
import requests_mock

from ..http import BackofficeHTTPException


@pytest.mark.parametrize('api_data', (
    [
        {
            'id': 100,
            'site_id': 128,
            'name': 'Камаз с пирожками',
            'offers_count': 450,
        },
        {
            'id': 234,
            'site_id': 256,
            'name': 'Ведро с тестами',
            'offers_count': 4,
        },
    ],
))
def test_product_offers_stat_url(api_data, backoffice):
    with requests_mock.mock() as m:
        m.get('http://testhost/products/offers_stat/?site_ids=128,256', status_code=200, json=api_data)

        assert backoffice.get_products_offers_stat([128, 256]) == api_data


def test_non_200_response_to_product_offers_stat_get(backoffice):
    with requests_mock.mock() as m:
        m.get('http://testhost/products/offers_stat/?site_ids=128,256', status_code=404)

        with pytest.raises(BackofficeHTTPException):
            assert backoffice.get_products_offers_stat([128, 256])



from setuptools import find_packages, setup

setup(
    name='gm-backoffice-client',
    version='1.2.4',
    description="Client for mtrl.ai orders API",
    keywords=[],
    url="https://github.com/gdml/backoffice-client/",
    author="Fedor Borshev",
    author_email="f@f213.in",
    license="MIT",
    packages=find_packages(),
    install_requires=[
        'requests',
        'jsonschema',
    ],
    include_package_data=True,
    zip_safe=False,
)
